"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@angular-devkit/core");
const schematics_1 = require("@angular-devkit/schematics");
const config_1 = require("@schematics/angular/utility/config");
const npm_script_1 = require("../utils/npm-script");
const package_1 = require("../utils/package");
const project_1 = require("../utils/project");
function add(options) {
    return schematics_1.chain([
        updateNpmConfig(),
        configure(options),
        createNewFiles(options),
    ]);
}
exports.default = add;
function updateNpmConfig() {
    return (host) => {
        package_1.moveDependencyFromDepsToDevDeps(host, 'angular-playground');
        npm_script_1.addNpmScriptToPackageJson(host, 'playground', 'angular-playground');
        return host;
    };
}
exports.updateNpmConfig = updateNpmConfig;
function addAppToWorkspaceFile(options, workspace, project, packageName) {
    const projectRoot = core_1.normalize(project.root);
    const sourceRoot = project_1.getSourceRoot(project.sourceRoot);
    const sourceRootParts = sourceRoot.split('/');
    const tsConfigPath = projectRoot === '' ? sourceRoot : projectRoot;
    const tsConfigPathParts = tsConfigPath.split('/');
    const newProject = {
        root: projectRoot,
        sourceRoot,
        projectType: 'application',
        architect: {
            build: {
                builder: '@angular-devkit/build-angular:browser',
                options: {
                    outputPath: constructPath(['dist', 'playground']),
                    index: constructPath([...sourceRootParts, 'index.html']),
                    main: constructPath([...sourceRootParts, 'main.playground.ts']),
                    polyfills: constructPath([...sourceRootParts, 'polyfills.ts']),
                    tsConfig: constructPath([...tsConfigPathParts, 'tsconfig.app.json']),
                    assets: [
                        constructPath([...sourceRootParts, 'favicon.ico']),
                        constructPath([...sourceRootParts, 'assets']),
                    ],
                    styles: [
                        constructPath([...sourceRootParts, `styles.${options.stylesExtension}`]),
                    ],
                    scripts: [],
                },
                configurations: {
                    production: {
                        fileReplacements: [
                            {
                                replace: constructPath([...sourceRootParts, 'environments', 'environment.ts']),
                                with: constructPath([...sourceRootParts, 'environments', 'environment.prod.ts']),
                            },
                        ],
                        optimization: true,
                        outputHashing: 'all',
                        sourceMap: false,
                        extractCss: true,
                        namedChunks: false,
                        aot: false,
                        extractLicenses: true,
                        vendorChunk: false,
                        buildOptimizer: false,
                    },
                },
            },
            serve: {
                builder: '@angular-devkit/build-angular:dev-server',
                options: {
                    browserTarget: 'playground:build',
                    port: 4201,
                },
            },
        },
    };
    return config_1.addProjectToWorkspace(workspace, packageName, newProject);
}
function configure(options) {
    return (host, context) => {
        const workspace = config_1.getWorkspace(host);
        const project = project_1.getProject(host, options);
        let stylesExtension = 'css';
        if (project.architect) {
            const mainStyle = project.architect.build.options.styles.find((path) => {
                return typeof path === 'string'
                    ? path.includes('/styles.')
                    : path.input.includes('/styles.');
            });
            if (mainStyle) {
                const mainStyleString = typeof mainStyle === 'string' ? mainStyle : mainStyle.input;
                stylesExtension = mainStyleString.split('.').pop();
            }
        }
        return schematics_1.chain([
            addAppToWorkspaceFile({ stylesExtension }, workspace, project, 'playground'),
        ])(host, context);
    };
}
function createNewFiles(options) {
    return (host, context) => {
        const project = project_1.getProject(host, options);
        const sourceRoot = project_1.getSourceRoot(project.sourceRoot);
        const angularPlaygroundJsonTemplateSource = schematics_1.apply(schematics_1.url('./files'), [
            schematics_1.filter(path => path.endsWith('angular-playground.json')),
            schematics_1.template(Object.assign({}, core_1.strings, { sourceRoots: [sourceRoot] })),
        ]);
        const playgroundMainTemplateSource = schematics_1.apply(schematics_1.url('./files'), [
            schematics_1.filter(path => path.endsWith('main.playground.ts')),
            schematics_1.template({}),
            schematics_1.move(sourceRoot),
        ]);
        return schematics_1.chain([
            schematics_1.branchAndMerge(schematics_1.mergeWith(angularPlaygroundJsonTemplateSource)),
            schematics_1.branchAndMerge(schematics_1.mergeWith(playgroundMainTemplateSource)),
        ])(host, context);
    };
}
const constructPath = (parts, isAbsolute = false) => {
    const filteredParts = parts.filter(part => !!part);
    return `${isAbsolute ? '/' : ''}${filteredParts.join('/')}`;
};
//# sourceMappingURL=index.js.map